@@ -10,26 +10,20 @@
  *
  * Author: Damian Waradzyn
  */
-size_t writeFunction(void *ptr, size_t size, size_t nmemb, void *stream) {
+size_t writeToFileStream(void *ptr, size_t size, size_t nmemb, void *stream) {
     fwrite(ptr, size, nmemb, (FILE *) stream);
     downloaded += nmemb * size;
     return nmemb * size;
-}
-;
+};
 
-int downloadUrl(char *url, char *filename, t_tile *tile) {
+int downloadAndStream(char *url, void* stream, void *writeToStream) {
     CURL *curl = curl_easy_init();
     CURLcode res;
-    FILE *destFile;
-//    long startTime = SDL_GetTicks();
     long http_code;
-//    fprintf(stderr, "downloading '%s' to '%s'\n", url, filename);
-
     if (curl == NULL) {
         fprintf(stderr, "downloadUrl failed: curl not initialized\n");
         return FALSE;
     }
-
     curl_easy_setopt(curl, CURLOPT_URL, url);
     if (options.referer != NULL) {
         curl_easy_setopt(curl, CURLOPT_REFERER, options.referer);
@@ -37,38 +31,48 @@ int downloadUrl(char *url, char *filename, t_tile *tile) {
     if (options.userAgent != NULL) {
         curl_easy_setopt(curl, CURLOPT_USERAGENT, options.userAgent);
     }
+    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, writeToStream);
+    curl_easy_setopt(curl, CURLOPT_WRITEDATA, stream);
+    res = curl_easy_perform(curl);
+    if (CURLE_OK == res) {
+         curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_code);
 
+         curl_easy_cleanup(curl);
+         if (http_code == 200) {
+             free(url);
+ //            fprintf(stderr, "download successfull, %ld\n", http_code);
+             return TRUE;
+         } else {
+             fprintf(stderr, "Received %ld status code for url: '%s'\n", http_code, url);
+             free(url);
+             return FALSE;
+         }
+     } else {
+         fprintf(stderr, "CURL error (url = '%s'): %s\n", url, curl_easy_strerror(res));
+     }
+
+     curl_easy_cleanup(curl);
+     free(url);
+     return FALSE;
+}
+
+
+int downloadAndSave(char *url, char *filename) {
+    FILE *destFile;
+//    long startTime = SDL_GetTicks();
+//    fprintf(stderr, "downloading '%s' to '%s'\n", url, filename);
     destFile = fopen(filename, "wb");
 
     if (destFile == NULL) {
         fprintf(stderr, "unable to open file '%s' for writing\n", filename);
-        curl_easy_cleanup(curl);
         return FALSE;
     }
-    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, writeFunction);
-    curl_easy_setopt(curl, CURLOPT_WRITEDATA, destFile);
-    res = curl_easy_perform(curl);
-    fclose(destFile);
-    if (CURLE_OK == res) {
-        curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_code);
-
-        curl_easy_cleanup(curl);
-        free(url);
-        if (http_code == 200) {
-//            fprintf(stderr, "download successfull, %ld\n", http_code);
-            return TRUE;
-        } else {
-            char * errorMessage = malloc(50);
-            sprintf(errorMessage, "HTTP %ld", http_code);
-            tile -> errorMessage = errorMessage;
-
-            remove(filename);
-            fprintf(stderr, "Received %ld status code\n", http_code);
-            return FALSE;
-        }
+    int result = downloadAndStream(url, destFile, &writeToFileStream);
+    if (!result) {
+        remove(filename);
     }
-    curl_easy_cleanup(curl);
-    free(url);
+    fclose(destFile);
+
 //    fprintf(stderr, "download '%s' took %ld ms\n", url, (SDL_GetTicks() - startTime));
-    return FALSE;
+    return result;
 }
